
EnableExplicit

;Nautilus Engine
;Version: 1.01
;Author: Mijikai

;Ortho Camera Example (tilemap)

XIncludeFile "nautilus.pbi"

Macro WindowEvents()
  Repeat
    win_msg = WindowEvent()
    If win_msg = #PB_Event_CloseWindow
      Break 2
    EndIf 
  Until win_msg = #Null
EndMacro

Procedure.i Demo(Width.i,Height.i,Title.s,ViewWidth.i,ViewHeight.i,FPS.i)
  Protected *engine.NAUTILUS_ENGINE
  Protected engine_delta.f
  Protected engine_fps.i
  Protected screen_center.NE_VECTOR
  Protected *atlas.NAUTILUS_TEXTURE
  Protected *tile.NAUTILUS_TILE
  Protected mouse.NE_VECTOR
  Protected *ortho.NAUTILUS_WORLD
  Protected cam.i
  Protected cam_scale.f
  Protected cam_origin.NE_VECTOR
  Protected cam_move.NE_VECTOR
  Protected cam_clamp.NE_CLAMP
  Protected px.i
  Protected py.i
  Protected tile_position.NE_VECTOR
  Protected tile_link.i
  Protected mouse_active.i
  Protected mouse_wheel.i
  Protected mouse_position.NE_VECTOR
  Protected mouse_index.NE_INDEX
  *engine = nautilusWindow(Title,Width,Height,#NE_WINDOW_NORMAL,ViewWidth,ViewHeight,FPS)
  If *engine
    *engine\RenderFilter()
    *engine\RenderAspect()
    *engine\RenderColor($400A05)          
    *engine\ScreenCenter(@screen_center) 
    *atlas = *engine\CreateTexture(?floor,#Null$)
    *tile = *atlas\CreateTile(2,1,0,0,32,32)
    *ortho = *engine\CreateWorld(300,300,32,32) ;<- create the ortho object/interface
    *ortho\TileMapCreate()                      ;<- create a tilemap (not needed a camera can also be used with a custom tilemap!) 
    For py = 0 To *ortho\TilesY() - 1
      For px = 0 To *ortho\TilesX() - 1
        *ortho\TileMapSet(px,py,Random(1))      ;<- set the link - the link stores informations related ti the tile
      Next
    Next
    cam = *ortho\CameraCreate(100,100,900,520)  ;<- create a camera
    cam_scale = 1                               ;<- initial scale/zoom factor
    cam_origin\x = 1600                          ;<- camera origin
    cam_origin\y = 1600
    *ortho\CameraOrigin(cam,@cam_origin)        ;<- set camera origin
    *engine\DrawTextResize(12,14)
    Repeat
      If *engine\InputKey(#VK_ESCAPE):Break:EndIf
      If *engine\InputKey(#VK_W):cam_move\y - 4:EndIf
      If *engine\InputKey(#VK_S):cam_move\y + 4:EndIf
      If *engine\InputKey(#VK_A):cam_move\x - 4:EndIf
      If *engine\InputKey(#VK_D):cam_move\x + 4:EndIf
      *engine\InputMouse(@mouse)
      mouse_active = *ortho\Mouse(cam,@mouse,@mouse_position,@mouse_index)  ;<- get the tile map mouse positions
      mouse_wheel = *engine\InputMouseWheel()
      If mouse_wheel
        If mouse_wheel > 0
          cam_scale + 1
        Else
          cam_scale - 1
        EndIf 
      EndIf 
      *ortho\CameraZoom(cam,1,3,@cam_scale)                                 ;<- zoom the camera
      cam_move\x * engine_delta
      cam_move\y * engine_delta
      *ortho\CameraMove(cam,@cam_move)                                      ;<- move the camera to new position
      *engine\RenderBegin(@engine_delta,@engine_fps)
      *engine\RenderPush()                                                  ;<- backup the current render matrix
      *engine\RenderClip(100,100,900,520)                                   ;<- clip the camera
      *ortho\CameraUpdate(cam,@cam_clamp)                                   ;<- update the camera
      For py = cam_clamp\min\y To cam_clamp\max\y
        For px = cam_clamp\min\x To cam_clamp\max\x
          *ortho\TileMapToScreen(cam,px,py,@tile_position,@tile_link)       ;<- translate all tiles that are visible
          If mouse_active
            If px = mouse_index\x And py = mouse_index\y
              *tile\DrawEx(tile_link,tile_position\x,tile_position\y,#False,#Null,#Null,$FF00FF)
            Else
              *tile\Draw(tile_link,tile_position\x,tile_position\y)
            EndIf 
          Else
            *tile\Draw(tile_link,tile_position\x,tile_position\y)
          EndIf
        Next
      Next
      *engine\RenderPop()                                                   ;<- restore render matrix
      *engine\RenderUnclip()                                                ;<- unclip the camera 
      *engine\DrawBox(100,100,900,520)
      *engine\DrawText(10,10,"Nautilus Engine",#False,#False,200)
      *engine\DrawText(10,25,"FPS " + Str(engine_fps),#False,#False,200)
      *engine\DrawText(10,40,"DLT " + StrF(engine_delta,2),#False,#False,200)
      *engine\DrawText(10,55,"Mouse Index: " + Str(mouse_index\x) + " x " + Str(mouse_index\y),#False,#False,200)
      *engine\RenderEnd()     
    Until *engine\WindowExit()
    *engine\Release()
  EndIf
  ProcedureReturn #Null
EndProcedure

Demo(1080,720,"Nautilus Engine (Ortho Camera)",1080,720,60)

End

DataSection
  floor:
  ;{ floor.png - Size: 236 Bytes
  !dw 05089h, 0474Eh, 00A0Dh, 00A1Ah, 00000h, 00D00h, 04849h, 05244h, 00000h, 04000h, 00000h, 02000h, 00208h, 00000h, 02D00h, 0E9FFh, 000D3h, 00000h, 049B3h, 04144h, 05854h, 0EDC3h, 0C198h, 00409h, 00C21h, 03345h, 0F4CBh, 055E0h, 06CF0h, 0E917h, 032C5h, 0C3ACh, 0925Eh, 0522Ah, 05D85h, 0C1ECh, 09645h, 07765h, 01660h, 0701Dh
  !dw 039F8h, 08129h, 05F07h, 03BC9h, 0CB64h, 0D339h, 0B5CAh, 09113h, 07AAAh, 067EFh, 043D0h, 0B308h, 07BF9h, 0A53Bh, 08694h, 04BD3h, 0FD29h, 08F3Ch, 00AFFh, 02220h, 0C631h, 07481h, 0DCE7h, 0F5FBh, 06F80h, 0CC66h, 06B5Ch, 0D53Dh, 0FCEAh, 02D07h, 0085Eh, 00080h, 03403h, 04D7Ah, 0D999h, 096AFh, 09C88h, 0F56Dh, 0BE21h, 0002Eh
  !dw 0AF33h, 003FDh, 06D5Fh, 08D08h, 00022h, 04002h, 0D0A3h, 03428h, 00B0Ah, 00021h, 00A34h, 0428Dh, 018A3h, 00462h, 083B8h, 00F46h, 0F734h, 07D47h, 0E66Ch, 06BA6h, 05554h, 03EC7h, 0034Ch, 0E5B6h, 054EEh, 0B6FEh, 07AFAh, 009FDh, 00798h, 0BD79h, 02B92h, 094DFh, 00000h, 00000h, 04549h, 0444Eh, 042AEh, 08260h
  ;}
EndDataSection

; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 4
; Folding = -
; EnableXP
; DisableDebugger